'use strict';
const Cc = Components.classes, Ci = Components.interfaces;
Components.utils.import('chrome://Stratiform/content/overlay.jsm');
Components.utils.import('resource://gre/modules/AddonManager.jsm');

var prefService = StratiformUtilities.prefService();
var prefBranch = StratiformUtilities.prefBranch();
var prefGeneral = StratiformUtilities.prefGeneral();
var ssService = StratiformUtilities.ssService();
var ioService = StratiformUtilities.ioService();
var mainWindow = StratiformUtilities.mainWindow();
var colorUtils = StratiformColorUtils;
var compatibilityMode = StratiformCompatibilityMode;

function $(id) {
  var i = document.getElementById(id);
  if (i)
    return i;
}

var Prefs = {
  get: function(key) {
    switch (prefBranch.getPrefType(key)) {
      case prefBranch.PREF_INT:
        return prefBranch.getIntPref(key);
      case prefBranch.PREF_BOOL:
        return prefBranch.getBoolPref(key);
      case prefBranch.PREF_STRING:
        return prefBranch.getCharPref(key);
    }
  } 
}

var debug = {
  copy: function() {
    Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper).copyString(debug.array);
  },
  init: function(action) {
    var prefs = ['disable-strict-compatibility','toolbar-style','appmenu-button-style','glyph-style','textfield-style','toolbar-bg','toolbar-radius','tabbar-style','inactivetab-bg','selectedtab-bg','tab-width','tab-height','addonbar-style','statuspopup-style','appmenu-bg','appmenu-style','appmenu-label','tabstyle-buttons','tabstyled-buttons-whitelist','leftsidestar','tabstitlebar','tabstitlebar-distance','disable-conditionalForward','thirdparty-customicons','show-in-firefoxMenu'];
    switch (action) {
      case 'support':
        debug.toggle('reset');
        var container = $('debug-list-settings');
        var subcontainer = $('debug-list-main');
        var enabledAddons = $('debug-list-enabledAddons');
        var containers = [container,subcontainer,enabledAddons];
        delete debug.array;
        debug.array = [];
        for (var i in containers) {
          if (containers[i].hasChildNodes()){
            while (containers[i].childNodes.length >= 1) {
              containers[i].removeChild(containers[i].firstChild);       
            } 
          }
        }
        AddonManager.getAddonByID("Stratiform@SoapySpew", function(addon) {
          var label = document.createElement('label');
          label.className = 'label-inverted list';
          label.setAttribute('value', 'Stratiform: ' + addon.version);
          subcontainer.appendChild(label);
          debug.array[0] = 'Stratiform: ' + addon.version;
        });
        var label = document.createElement('label');
        label.className = 'label-inverted list';
        label.setAttribute('value', 'Firefox: ' + Application.version);
        container.appendChild(label);
        debug.array.push('\n' + 'Firefox: ' + Application.version);
        var label = document.createElement('label');
        label.className = 'label-inverted list';
        label.setAttribute('value', 'compatibility mode: ' + compatibilityMode._enabled);
        container.appendChild(label);
        debug.array.push('\n' + 'compatibility mode: ' + compatibilityMode._enabled);
        for (var i in prefs) {
          var pref = Prefs.get(prefs[i]);
          if (pref) {
            var label = document.createElement('label');
            label.className = 'label-inverted list';
            label.setAttribute('value', prefs[i] + ': ' + pref);
            container.appendChild(label);
            debug.array.push('\n' + prefs[i] + ': ' + pref);
          }
        }
        AddonManager.getAddonsByTypes(["extension"], function(extensions) {
          debug.array.push('\n\------ Enabled addons ------');
          for (var i = 0; i < extensions.length; i++) {
            if (extensions[i].isActive && extensions[i].id != "Stratiform@SoapySpew") {
              var label = document.createElement('label');
              label.className = 'label-inverted list';
              label.setAttribute('value', extensions[i].name + ': ' + extensions[i].version);
              enabledAddons.appendChild(label);
              debug.array.push('\n' + extensions[i].name + ': ' + extensions[i].version);
            }
          }
        });
        break;
      case 'reset':
        for (var i in prefs)
          prefBranch.clearUserPref(prefs[i]);
        $('tabstitlebar-textbox').value = Prefs.get('tabstitlebar-distance');
        tabsInTitlebar.toggleVisibility();
        tabbuttonExclude.toggleVisibility();
        scales.restore();
    }
  },
  toggle: function(action) {
    var normal = $('debug-normal-list');
    var details = $('debug-vbox');
    var link = $('debug-toggle-link');
    switch (action) {
      case 'reset':
        normal.hidden = false;
        details.hidden = true;
        link.hidden = false;
        break;
      case 'more':
        normal.hidden = true;
        details.hidden = false;
        link.hidden = true;
    }
  }
}

function disableIncompatSettings() {
  var options = ['tab-width-textbox','tab-height-textbox','tabstyle-button-checkbox','tab-style-menulist','tab-style-label','selected-tab-tab','inactive-tab-tab'];
  for (var i in options)
    $(options[i]).setAttribute('disabled',true);
}

function init() {
  if (compatibilityMode._enabled) {
    var detAddons = $('detectedAddons');
    var incompatList = compatibilityMode.incompatList;
    var enabledAddons = compatibilityMode.enabledAddons;
    if (!compatibilityMode.disableCheck) {
      for (var i in incompatList) {
        if (enabledAddons.indexOf(incompatList[i]) !=-1) {
          disableIncompatSettings();
          if (detAddons) {
            $('compatibility-tab').removeAttribute('hidden');
            mainWindow.AddonManager.getAddonByID(incompatList[i], function(addon) {
              var label = document.createElement('description');
              label.setAttribute('class','label-general label-link');
              if (addon.homepageURL != 'undefined')
                label.setAttribute('onclick', 'mainWindow.switchToTabHavingURI("' + addon.homepageURL + '",true)');
              label.setAttribute('value',addon.name);
              detAddons.appendChild(label);
            });
          }
        }
      }  
    }
    $('compatibility-tab').removeAttribute('hidden');
    $('stratiform-main-radiogroup').value = 4;
    switchDeck(4);
  } else
    switchDeck(3);
  if (!compatibilityMode._enabled && compatibilityMode.disableCheck)
    $('compatibility-tab').removeAttribute('hidden');
  tabbuttonExclude.toggleVisibility();
  compatCheckbox();
  tabsInTitlebar.toggleVisibility();
  uiHighlight.init();
  scales.restore();
  $('tabstitlebar-textbox').value = Prefs.get('tabstitlebar-distance');
}

function uninit() {}

var tabbuttonExclude = {
  build: function() {
    this.vbox = $('tabbutton-exclude-list');
    this.pref = prefBranch.getCharPref('tabstyled-buttons-whitelist').split(',');
    var list = mainWindow.document.getElementById('TabsToolbar').getElementsByClassName('toolbarbutton-1');
    for (var i = 0; i < list.length; i ++) {
      var chk = document.createElement('checkbox');
      //chk.className = 'checkbox-flipped';
      if (list[i].hasAttribute('label'))
        chk.setAttribute('label',list[i].label);
      else
        chk.setAttribute('label',list[i].id);
      chk.setAttribute('element',list[i].id);
      if (this.pref.indexOf(list[i].id) !=-1)
        chk.setAttribute('checked',true);
      this.vbox.appendChild(chk);
    }
  },
  buildArray: function() {
    var array = [];
    var vbox = this.vbox.getElementsByTagName('checkbox');
    for (var i = 0; i < vbox.length; i ++) {
      if (vbox[i].hasAttribute('checked'))
        array.push(vbox[i].getAttribute('element'));
    }
    prefBranch.setCharPref('tabstyled-buttons-whitelist',array);
    // Remove checkboxes from list
    var chk_list = this.vbox;
    if (chk_list.hasChildNodes()) {
      while (chk_list.childNodes.length >= 1) {
        chk_list.removeChild(chk_list.firstChild);       
      }
    }
    // Toggle pref to reload
    prefBranch.setBoolPref('tabstyle-buttons',false);
    prefBranch.setBoolPref('tabstyle-buttons',true);
  },
  toggleVisibility: function() {
    var chk = $('tabstyle-button-checkbox');
    var exc = $('tabbutton-exclude-hbox');
    exc.setAttribute('hidden',!chk.checked);
  }
}

function compatCheckbox() {
  var chk = $('compatibility-checkbox');
  var exc = $('compatibility-hbox');
  exc.hidden = !chk.checked;
}

var tabsInTitlebar = {
  togglePref: function(value) {
    prefBranch.setBoolPref('tabstitlebar',false);
    prefBranch.setIntPref('tabstitlebar-distance',value);
    prefBranch.setBoolPref('tabstitlebar',true);
  },
  toggleVisibility: function() {
    var chk = $('tabstitlebar-checkbox');
    var exc = $('tabstitlebar-distance-vbox');
    exc.hidden = !chk.checked;
  }
}

function acceptDialog(event) {
  var deck = popup.getDeck();
  if (deck.selectedIndex == 1 && event.keyCode == 13) {
    var pref = popup._pref;
    var type = popup._type;
    if (type == 'hex' || type == 'hsl' || type == 'rgb')
      popup.importColor(pref,type);
  }
}

var popup = {
  _pref: null,
  _type: null,
  getDeck: function() {
    return $('page_deck');
  },
  delval: function() {
    delete popup._pref;
    delete popup._type;
  },
  show: function(type,pref,action) {
    var hex_con = $('hex-container');
    var rgb_con = $('rgb-container');
    var hsl_con = $('hsl-container');
    var preset_con = $('preset-name-container');
    var ok = $('popupOK');
    var cancel = $('popupCancel');
    var popup_deck = $('popup-deck');
    this.getDeck().selectedIndex = 1;
    popup_deck.selectedIndex = 0;
    $('warningbox-hex').hidden = true;
    $('warningbox-rgb').hidden = true;
    $('warningbox-hsl').hidden = true;
    switch (type) {
      case 'tabstyled-buttons':
        if (action != 'hide') {
          popup_deck.selectedIndex = 1;
          tabbuttonExclude.build();
          $('exclude-ok-button').focus();
        }
        break;
      case 'reset-options':
        if (action != 'hide') {
          popup_deck.selectedIndex = 2;
          $('reset_options-ok-button').focus();
        }
        break;
      case 'report-bug':
        if (action != 'hide') {
          popup_deck.selectedIndex = 3;
          debug.init('support');
          $('report_bug-ok-button').focus();
        }
        break;
      case 'hex':
        if (action != 'hide') {
          hex_con.hidden = false;
          $('hexTextbox').focus();
        }
        else
          hex_con.hidden = true;
        break;
      case 'rgb':
        if (action != 'hide') {
          rgb_con.hidden = false;
          $('rTextBox').focus();
        }
        else
          rgb_con.hidden = true;
        break;
      case 'hsl':
        if (action != 'hide') {
          hsl_con.hidden = false;
          $('hTextBox').focus();
        }
        else
          hsl_con.hidden = true;
        break;
      case 'new-preset':
        if (action != 'hide') {
          popup_deck.selectedIndex = 4;
          $('preset-ok-button').focus();
        }
        break;
      case 'preset':
        if (action != 'hide') {
          preset_con.hidden = false;
          ok.setAttribute('oncommand','ColorliciousOptions.customPreset("new");');
          $('presetTextbox').focus();
        }
        else
          preset_con.hidden = true;
        break;
    }
    if (type != 'preset' && type !='tabstyled-buttons' && type !='reset-options')
      ok.setAttribute('oncommand','popup.importColor("' + pref + '","' + type + '");');
    cancel.setAttribute('oncommand','popup.show("' + type + '","null","hide")');
    this._pref = pref;
    this._type = type;
    if (action == 'hide') {
      this.getDeck().selectedIndex = 0;
      delete this._pref;
      delete this._type; 
    }
 },
 importColor: function(pref,type) {
   var s_hue = $('sliderHue-' + pref).firstChild;
   var s_sat = $('sliderSaturation-' + pref).firstChild;
   var s_lig = $('sliderLightness-' + pref).firstChild;
   var s_alp = $('sliderAlpha-' + pref).firstChild;
   switch (type) {
     case 'hsl':
       var h = $('hTextBox');
       var s = $('sTextBox');
       var l = $('lTextBox');
       if ((h.value <= 360 && h.value >= 0)
           && (s.value <=100 && s.value >= 0)
           && (l.value <=100 && l.value >= 0)) {
         s_hue.value = h.value;
         s_sat.value = s.value;
         s_lig.value = l.value;
         s_alp.value = 100;
         this.show(type,'null','hide');
         this.delval();
       } else
           $('warningbox-hsl').hidden = false;
       h.value = '';
       s.value = '';
       l.value = '';
       break;
       case 'hex':
         var box = $('hexTextbox');
         var hex = box.value;
         var validHex = /^(#)?([0-9a-fA-F]{3})([0-9a-fA-F]{3})?$/;
         if (validHex.test(hex)) {
           var rgb = colorUtils.hexToRgb(hex);
           var hsl = colorUtils.rgbToHsl(rgb[0],rgb[1],rgb[2]);
           s_hue.value = hsl[0];
           s_sat.value = hsl[1];
           s_lig.value = hsl[2];
           s_alp.value = 100;
           this.show(type,'null','hide');
           this.delval();
         } else
             $('warningbox-hex').hidden = false;
         box.value = '';
       break;
       case 'rgb':
         var r = $('rTextBox');
         var g = $('gTextBox');
         var b = $('bTextBox');
         if (r.value <= 255 && r.value >= 0
             && g.value <= 255 && g.value >= 0
             && b.value <= 255 && b.value >= 0) {
           var hsl = colorUtils.rgbToHsl(r.value,g.value,b.value);
           s_hue.value = hsl[0];
           s_sat.value = hsl[1];
           s_lig.value = hsl[2];
           s_alp.value = 100;
           this.show(type,'null','hide');
           this.delval();
         } else
             $('warningbox-rgb').hidden = false;
         r.value = '';
         g.value = '';
         b.value = '';
       break;
   }
 }
}

var uiHighlight = {
  set: function(val) {
    if (val == '9') {
      prefBranch.clearUserPref('uihighlight');
      document.documentElement.removeAttribute('color');
    }
    else {
      prefBranch.setIntPref('uihighlight',val);
      document.documentElement.setAttribute('color',val);
    }
  },
  init: function() {
    var pref = prefBranch.getIntPref('uihighlight');
    if (pref != '9') {
      document.documentElement.setAttribute('color',pref);
      $('highlight-radiogroup').value = pref;
    }
  }
}

var feedback = {
  bug: function() {
    mainWindow.Stratiform.openTab('http://code.google.com/p/stratiform/issues/list');
  },
  positive: function() {
    mainWindow.Stratiform.openTab('mailto:StratiformFeedback@gmail.com?subject=Stratiform Made Me Happy Because...');
  },
  negative: function() {
    mainWindow.Stratiform.openTab('mailto:StratiformFeedback@gmail.com?subject=Stratiform Made Me Sad Because...');
  },
  request: function() {
    mainWindow.Stratiform.openTab('mailto:StratiformFeedback@gmail.com?subject=I\'d like Stratiform to...');
  }
}

function switchDeck(val) {
  var deck = $('main-deck');
  deck.selectedIndex = val;
  for (var i=0; i < 6;i ++) {
    $('mainDeck-' + i).removeAttribute('transitioned');
  }
  $('mainDeck-' + val).setAttribute('transitioned',true);
}

var scales = {
  build: function(scale,B) {
    var h = $('sliderHue-' + scale);
    var s = $('sliderSaturation-' + scale);
    var l = $('sliderLightness-' + scale);
    var a = $('sliderAlpha-' + scale);
    h.value = h.firstChild.value;
    s.value = s.firstChild.value;
    l.value = l.firstChild.value;
    a.value = a.firstChild.value;
    s.style.backgroundImage = '-moz-linear-gradient(left, hsla(' + h.value  + ',0%,50%,1), hsla(' + h.value  + ',100%,50%,1))';
    l.style.backgroundImage = '-moz-linear-gradient(left, hsla(' + h.value  + ',' + s.value + '%,0%,1), hsla(' + h.value  + ',' + s.value + '%,50%,1), hsla(' + h.value  + ',' + s.value + '%,100%,1))';
    a.style.backgroundImage = '-moz-linear-gradient(left, hsla(' + h.value  + ',' + s.value + '%,' + l.value + '%,0), hsla(' + h.value  + ',' + s.value + '%,' + l.value + '%,1)),url(chrome://stratiform/skin/options/trans-bg.png)';
    var hsl = {};
    hsl.H = h.value;
    hsl.S = s.value;
    hsl.L = l.value;
    hsl.A = a.value;
	setTimeout(function(){prefBranch.setCharPref(scale,JSON.stringify(hsl))},100);
  },
  restore: function() {
    var scales = ['toolbar-bg','selectedtab-bg','inactivetab-bg','appmenu-bg'];
    var hsla;
    for (var i in scales) {
      var pref = Prefs.get(scales[i]);
      if (pref) {
        var hsla = JSON.parse(pref);
        $('sliderHue-' + scales[i]).firstChild.setAttribute('value',hsla.H);
        $('sliderSaturation-' + scales[i]).firstChild.setAttribute('value',hsla.S);
        $('sliderLightness-' + scales[i]).firstChild.setAttribute('value',hsla.L);
        $('sliderAlpha-' + scales[i]).firstChild.setAttribute('value',hsla.A);
      } else
      if (!pref) {
        this.reset(scales[i]);
      }
    }
  },
  reset: function(scale) {
    var h = $('sliderHue-' + scale);
    var s = $('sliderSaturation-' + scale);
    var l = $('sliderLightness-' + scale);
    var a = $('sliderAlpha-' + scale);
    h.firstChild.value = 180;
    s.firstChild.value = 50;
    l.firstChild.value = 50;
    a.firstChild.value = 100;
    setTimeout(function(){prefBranch.clearUserPref(scale)},105);
  },
  buildFromPreset: function(value,scale) {
    switch (value) {
      case '0': var h = 210; var s = 95; var l = 95; break;
      case '1': var h = 216; var s = 52; var l = 57; break;
      case '2': var h = 180; var s = 61; var l = 60; break;
      case '3': var h = 113; var s = 47; var l = 56; break;
      case '4': var h = 85; var s = 68; var l = 64; break;
      case '5': var h = 52; var s = 95; var l = 76; break;
      case '6': var h = 37; var s = 100; var l = 67; break;
      case '7': var h = 0; var s = 66; var l = 58; break;
      case '8': var h = 324; var s = 100; var l = 75; break;
      case '9': var h = 305; var s = 89; var l = 93; break;
    }
    $('sliderHue-' + scale).firstChild.value = h;
    $('sliderSaturation-' + scale).firstChild.value = s;
    $('sliderLightness-' + scale).firstChild.value = l;
    $('sliderAlpha-' + scale).firstChild.value = 100;
  },
}

var globalPreset = {
  change: function(val) {
    switch (val) {
      case '0':
        var prefs = ['addonbar-style','glyph-style','tabbar-style','textfield-style','toolbar-style'];
        for (var i in prefs)
          prefBranch.setCharPref(prefs[i],'s40');
        break;
      case '1':
        var prefs = ['glyph-style','textfield-style','toolbar-style'];
        var prefs_reset  = ['addonbar-style','tabbar-style'];
        for (var i in prefs)
          prefBranch.setCharPref(prefs[i],'7native');
        for (var i in prefs_reset)
          prefBranch.clearUserPref(prefs_reset[i]);
        prefBranch.setCharPref('statuspopup-style','native');
        break;
      case '2':
        var prefs = ['glyph-style','tabbar-style','textfield-style','toolbar-style'];
        for (var i in prefs)
          prefBranch.setCharPref(prefs[i],'wave');
        break;
      case '3':
        var prefs = ['appmenu-button-style','glyph-style','tabbar-style','textfield-style','toolbar-style'];
        for (var i in prefs)
          prefBranch.setCharPref(prefs[i],'soft7');
        prefBranch.setCharPref('statuspopup-style','native');
        break;
    }
  },
}