'use strict';
(function() {

Components.utils.import('chrome://stratiform/content/overlay.jsm');
Components.utils.import('resource://gre/modules/AddonManager.jsm');
Components.utils.import('resource://gre/modules/Services.jsm');

let prefService = StratiformUtilities.prefService();
let prefBranch = StratiformUtilities.prefBranch();
let prefGeneral = StratiformUtilities.prefGeneral();
let ssService = StratiformUtilities.ssService();
let ioService = StratiformUtilities.ioService();
let mainWindow = StratiformUtilities.mainWindow();
let colorUtils = StratiformColorUtils;
let compatibilityMode = StratiformCompatibilityMode;


let BaseURI = ioService.newURI('chrome://stratiform/skin/', null, null);

function $(id) {
  try {
    var i = document.getElementById(id);
    if (i)
      return i;
    else
      return mainWindow.document.getElementById(id);
  } catch(e) {}
}

var sheetService = {
  namespace: '@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");@-moz-document url("chrome://browser/content/browser.xul") {',
  enable: function(sheet) {
    if (!ssService.sheetRegistered(sheet, ssService.USER_SHEET))
      ssService.loadAndRegisterSheet(sheet, ssService.USER_SHEET);
  },
  disable: function(sheet) {
    if (ssService.sheetRegistered(sheet, ssService.USER_SHEET))
      ssService.unregisterSheet(sheet, ssService.USER_SHEET);
  },
  load: function(url, base) {
    var sheet = ioService.newURI(url, null, base);
    this.enable(sheet);
    return sheet;
  },
  getEmbedCSSURL: function(css) {
    return 'data:text/css;charset=utf-8,' + encodeURIComponent(this.namespace + css);
  }
};

const appmenuLabel = {
  change: function(newval) { 
    try {
      var utf8Converter = Cc["@mozilla.org/intl/utf8converterservice;1"].getService(Ci.nsIUTF8ConverterService);
      var button = $('appmenu-button');
      var newval = utf8Converter.convertURISpecToUTF8(newval, "UTF-8");
      if (button)
        button.setAttribute('label', newval ? newval : $('bundle_brand').getString('brandShortName'));
    } catch(e) {}
  }
};

const toolbarBackgroundColor = {
  _ref: null,
  change: function(newval) {
    if (this._ref) {
      sheetService.disable(this._ref);
      delete this._ref;
    }
    if (newval) {
      try {
        var values = JSON.parse(newval);
        if (values) {
          var H = values.H;
          var S = values.S;
          var L = values.L;
          var A = values.A;
        }
    this._ref = newval ? sheetService.load(sheetService.getEmbedCSSURL(
'/* default style */\
/* normal state */\
window:not([stratiform-toolbar-style]):not([chromehidden~=toolbar]) #navigator-toolbox[iconsize=large][mode=icons] > :-moz-any(#nav-bar[currentset*="unified-back-forward-button,urlbar-container"],#nav-bar:not([currentset])) > #unified-back-forward-button > #back-button .toolbarbutton-icon,\
#main-window:not([stratiform-toolbar-style]) #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1[type=menu-button]:not([buttonover]):-moz-any(:hover, [open]) > .toolbarbutton-menubutton-button > .toolbarbutton-icon,\
#main-window:not([stratiform-toolbar-style]) #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1[type=menu-button][buttonover] > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A*.75) + '!important;\
}\
\
/* hover state */\
#main-window:not([stratiform-toolbar-style]) #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:not([buttonover]):hover > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon,\
#main-window:not([stratiform-toolbar-style]) #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button):not([disabled]):not([checked]):not([open]):not(:active):hover > .toolbarbutton-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
\
/* pressed state */\
#main-window:not([stratiform-toolbar-style]) #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:not([buttondown])[open] > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon,\
#main-window:not([stratiform-toolbar-style]) #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button):not([disabled]):-moz-any([checked], [open], :hover:active) > .toolbarbutton-icon {\
  background-color:' + colorUtils.HSLA(H,S,L-10,A) + '!important;\
}\
\
/* strata40 style */\
/* normal state */\
#main-window[stratiform-toolbar-style=s40] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button) > .toolbarbutton-icon,\
#main-window[stratiform-toolbar-style=s40] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1 > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
\
/* hover state */\
#main-window[stratiform-toolbar-style=s40] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:not([buttonover]):hover > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon,\
#main-window[stratiform-toolbar-style=s40] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button):not([disabled]):not([checked]):not([open]):not(:active):hover > .toolbarbutton-icon {\
  background-color:' + colorUtils.HSLA(H,S,L+12,A) + '!important;\
}\
\
/* pressed state */\
#main-window[stratiform-toolbar-style=s40] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:not([buttondown])[open] > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon,\
#main-window[stratiform-toolbar-style=s40] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button):not([disabled]):-moz-any([checked], [open], :hover:active) > .toolbarbutton-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
\
/* vestige style */\
/* normal state */\
#main-window[stratiform-toolbar-style=untitled] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button) > .toolbarbutton-icon,\
#main-window[stratiform-toolbar-style=untitled] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1 > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
\
/* hover state */\
#main-window[stratiform-toolbar-style=untitled] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:not([buttonover]):hover > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon,\
#main-window[stratiform-toolbar-style=untitled] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button):not([disabled]):not([checked]):not([open]):not(:active):hover > .toolbarbutton-icon {\
  background-color:' + colorUtils.HSLA(H,S,L+20,A) + '!important;\
  border-color:' + colorUtils.HSLA(H,S,L+25,A) + '!important;\
  box-shadow: 0 0 1px white inset,\
              1px 0 0 rgba(0,0,0,.01),\
              -1px 0 0 rgba(0,0,0,.01),\
              0 1px 0 rgba(0,0,0,.05),\
              0 0 5px ' + colorUtils.HSLA(H,S,L+25,A) + '!important;\
}\
\
/* pressed state */\
#main-window[stratiform-toolbar-style=untitled] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:not([buttondown])[open] > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon,\
#main-window[stratiform-toolbar-style=untitled] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button):not([disabled]):-moz-any([checked], [open], :hover:active) > .toolbarbutton-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
\
/* windows 7 style */\
/* normal state */\
#main-window[stratiform-toolbar-style="7native"] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button):not(:-moz-any(:not([disabled=true]):hover, :not([disabled=true]):hover:active, [open], [checked], [buttonover], [buttondown])) > .toolbarbutton-icon,\
#main-window[stratiform-toolbar-style="7native"] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:-moz-any([buttonover], :not(:hover)) > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
  border-color:' + colorUtils.HSLA(H,S,L-5,A) + '!important;\
}\
\
/* soft7 style */\
/* normal state */\
#main-window[stratiform-toolbar-style=soft7] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button) > .toolbarbutton-icon,\
#main-window[stratiform-toolbar-style=soft7] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1 > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
\
/* rubix style */\
/* normal state */\
#main-window[stratiform-toolbar-style=rubix] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button) > .toolbarbutton-icon,\
#main-window[stratiform-toolbar-style=rubix] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1 > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
\
/* wave style */\
/* normal state */\
#main-window[stratiform-toolbar-style=wave] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar :-moz-any(.toolbarbutton-1, .toolbarbutton-menubutton-button):not(:-moz-any(:not([disabled=true]):hover, :not([disabled=true]):hover:active, [open], [checked], [buttonover], [buttondown])) > .toolbarbutton-icon,\
#main-window[stratiform-toolbar-style=wave] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:-moz-any([buttonover], :not(:hover)) > .toolbarbutton-menubutton-dropmarker > .dropmarker-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
}\
    '), null) : null;
      } catch(e) {}
    }
  }
};

const toolbarBorderRadius = {
  _ref: null,
  change: function(newval) {
    if (this._ref) {
      sheetService.disable(this._ref);
      delete this._ref;
    }
    if (newval != 2) this._ref = newval ? sheetService.load(sheetService.getEmbedCSSURL(
'#navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:not(#back-button):not(#forward-button) > .toolbarbutton-icon,\
#navigator-toolbox[iconsize=large][mode=icons] > #nav-bar > toolbarbutton[type=menu-button] > .toolbarbutton-menubutton-button:-moz-locale-dir(ltr) > .toolbarbutton-icon,\
#navigator-toolbox[iconsize=large][mode=icons] > #nav-bar > toolbarbutton[type=menu-button] > .toolbarbutton-menubutton-dropmarker:-moz-locale-dir(rtl) > .dropmarker-icon,\
#nav-bar:not([currentset*="unified-back-forward-button,urlbar-container"]) #urlbar,\
#nav-bar[currentset*="unified-back-forward-button,urlbar-container"] #urlbar:-moz-locale-dir(rtl) {\
  border-top-left-radius:' + newval + 'px!important;\
  border-bottom-left-radius:' + newval + 'px!important;\
}\
#navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1:not(#back-button):not(#forward-button) > .toolbarbutton-icon,\
#navigator-toolbox[iconsize=large][mode=icons] > #nav-bar > toolbarbutton[type=menu-button] > .toolbarbutton-menubutton-button:-moz-locale-dir(rtl) > .toolbarbutton-icon,\
#navigator-toolbox[iconsize=large][mode=icons] > #nav-bar > toolbarbutton[type=menu-button] > .toolbarbutton-menubutton-dropmarker:-moz-locale-dir(ltr) > .dropmarker-icon,\
#nav-bar:not([currentset*="unified-back-forward-button,urlbar-container"]) #urlbar,\
#nav-bar[currentset*="unified-back-forward-button,urlbar-container"] #urlbar:-moz-locale-dir(ltr) {\
  border-top-right-radius:' + newval + 'px!important;\
  border-bottom-right-radius:' + newval + 'px!important;\
}\
#main-window[stratiform-toolbar-style=rubix] #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar .toolbarbutton-1 > .toolbarbutton-icon,\
#main-window:not([stratiform-toolbar-style=soft7]) #navigator-toolbox[iconsize=large][mode=icons] > #nav-bar:not([currentset*="unified-back-forward-button,urlbar-container"]) > #unified-back-forward-button > .toolbarbutton-1 > .toolbarbutton-icon,\
#aios-sbhtoolbar .toolbarbutton-1,\
#aios-toolbar .toolbarbutton-1,\
#main-window #searchbar > .searchbar-textbox {\
  border-radius:' + newval + 'px!important;\
}\
}\
    '), null) : null;
  }
};

const appmenuColor = {
  _ref: null,
  change: function(newval) {
    if (this._ref) {
      sheetService.disable(this._ref);
      delete this._ref;
    }
    if (newval) {
      var values = JSON.parse(newval);
      if (values) {
        var H = values.H;
        var S = values.S;
        var L = values.L;
        var A = values.A;
      }
      var textcolor = colorUtils.colorBrightness(H,S,L) >= 50 ?
        '#appmenu-button {\
          color: rgba(0,0,0,.8) !important;\
          text-shadow: 0 0 1px rgba(255,255,255,.7),\
                       0 1px 1.5px rgba(255,255,255,.5) !important;\
        }\
        #appmenu-button > .button-box > .button-menu-dropmarker {\
          list-style-image: url(chrome://stratiform/skin/appmenu-button/appmenu-dropmarker.png) !important;\
        }\
        #appmenu-button > .button-box > .button-text {\
          opacity: .9999 !important;\
        }' : '';
      this._ref = newval ? sheetService.load(sheetService.getEmbedCSSURL(
        "#main-window:not([privatebrowsingmode]):not([stratiform-appmenubutton-style='soft7']) #appmenu-button {\
          -moz-border-left-colors: rgba(255,255,255,.6) rgba(0,0,0,.66) !important;\
          -moz-border-bottom-colors: rgba(255,255,255,.6) rgba(0,0,0,.66) !important;\
          -moz-border-right-colors: rgba(255,255,255,.6) rgba(0,0,0,.66) !important;\
          background-color: transparent !important;\
          background-image:-moz-linear-gradient(hsla("+ (H+5) + "," + (S*.9) + "%," + (L*2*.64) + "%," + (A/100) +"), hsla(" + (H-5) + ","+ (S*.91) + "%," + (L*2*.44) +"%," + (A/100) + ") 95%)!important;\
        }\
        #main-window:not([privatebrowsingmode]):not([stratiform-appmenubutton-style='soft7']) #appmenu-button:not([open]):hover {\
          background-image:-moz-radial-gradient(center bottom, farthest-side, hsla(" + (H+25) + "," + (S*.96) + "%," + (L*2*.67) +"%,.5)10%,hsla(" + (H+25) + "," + (S*.96) + "%," + (L*.67) +"%,0)70%)," + "-moz-radial-gradient(center bottom, farthest-side, hsla(" + (H+3) + "," + S + "%," + (L*2*.46) + "%,1),hsla(" + (H+10) + "," + S + "%," + (L*2*.84) + "%,0)),-moz-linear-gradient(hsla(" + (H+3) + "," + (S*.91) + "%," + (L*2*.62) + "%," + (A/100) +"), hsla(" + (H-10) + ","+ S + "%," + (L*2*.41) +"%," + (A/100) + ") 95%)!important;\
          box-shadow: 0 1px hsla(0,0%,100%,.1) inset, 0 0 1.5px 1px hsla(" + (H+17) +","+ (S*.89) + "%," + (L*2*.82) + "%,.7)inset,0 -1px 0 hsla(" + (H+17) +","+(S*.89) +"%," + (L*2*.82) +"%,.5) inset!important;\
        }\
        #main-window:not([privatebrowsingmode])[stratiform-appmenubutton-style='soft7'] #appmenu-button {\
          background-color:" + colorUtils.HSLA(H,S,L,A) + "!important;\
        }\
        }\
      " + textcolor), null) : null;
    }
  }
};

const tabWidth = {
  _ref: null,
  change: function(newval) {
    if (this._ref) {
      sheetService.disable(this._ref);
      delete this._ref;
    }
    if (newval != 250) this._ref = newval ? sheetService.load(sheetService.getEmbedCSSURL(
'.tabbrowser-tab[fadein]:not([faviconized=true]):not([pinned]){\
  max-width:' + newval + 'px!important;\
  min-width: 100px!important;\
  width: 0!important;\
}\
}\
    '), null) : null;
  }
};

const tabHeight = {
  _ref: null,
  change: function(newval) {
    if (this._ref) {
      sheetService.disable(this._ref);
      delete this._ref;
    }
    if (newval != 26) this._ref = newval ? sheetService.load(sheetService.getEmbedCSSURL(
 '.tabbrowser-tab .tab-stack,\
 #main-window:not([stratiform-compatibility-mode]) #TabsToolbar[stratiform-tabstyle-buttons] .toolbarbutton-1[stratiform-tabstyled] .toolbarbutton-icon {\
  height:' + newval + 'px !important;\
  min-height:' + newval + 'px !important;\
}\
.tabs-newtab-button > .toolbarbutton-icon,\
#TabsToolbar[stratiform-tabstyle-buttons] .toolbarbutton-1[stratiform-tabstyled]:not(#alltabs-button) > .toolbarbutton-icon,\
#TabsToolbar[stratiform-tabstyle-buttons] .toolbarbutton-1[stratiform-tabstyled]:not(#alltabs-button) > .toolbarbutton-menubutton-button .toolbarbutton-icon {\
  padding-top: ' + ((newval - 18) / 2 - .5) + 'px !important;\
  padding-bottom: ' + ((newval - 18) / 2 - .5) + 'px !important;\
}\
#TabsToolbar[stratiform-tabstyle-buttons] > #alltabs-button[type="menu"][stratiform-tabstyled] > .toolbarbutton-icon {\
  padding-top: ' + ((newval - 7) / 2 - .5) + 'px !important;\
  padding-bottom: ' + ((newval - 7) / 2 - .5) + 'px !important;\
}\
}\
    '), null) : null;
  }
};

const inactiveTabColor = {
  _ref: null,
  change: function(newval) {
    if (this._ref) {
      sheetService.disable(this._ref);
      delete this._ref;
    }
    if (newval) {
      try {
        var values = JSON.parse(newval);
        if (values) {
          var H = values.H;
          var S = values.S;
          var L = values.L;
          var A = values.A;
        }
      var textcolor = colorUtils.colorBrightness(H,S,L) <= 50 ?
        '.tabbrowser-tab:not([selected]):not(:-moz-lwtheme) .tab-text {\
          color: white !important;\
          text-shadow: none !important;\
      }' : '';
    this._ref = newval ? sheetService.load(sheetService.getEmbedCSSURL(
'#main-window .tabbrowser-tab:not([selected]) > .tab-stack,\
#main-window #TabsToolbar[stratiform-tabstyle-buttons] .toolbarbutton-1[stratiform-tabstyled] > .toolbarbutton-icon,\
#main-window #TabsToolbar .tabs-newtab-button > .toolbarbutton-icon {\
  background-color:' + colorUtils.HSLA(H,S,L,A) + '!important;\
}\
#main-window .tabbrowser-tab:not([selected]):hover > .tab-stack,\
#main-window #TabsToolbar[stratiform-tabstyle-buttons] .toolbarbutton-1[stratiform-tabstyled]:not([disabled]):hover > .toolbarbutton-icon,\
#main-window #TabsToolbar .tabs-newtab-button:hover > .toolbarbutton-icon {\
  background-color:' + colorUtils.HSLA(H,S - 5,L + 10,A) + '!important;\
}\
}\
    ' + textcolor), null) : null;
      } catch(e) {}
    }
  }
};

const activeTabColor = {
  _ref: null,
  change: function(newval) {
    if (this._ref) {
      sheetService.disable(this._ref);
      delete this._ref;
    }
    if (newval) {
      try {
        var values = JSON.parse(newval);
        if (values) {
          var H = values.H;
          var S = values.S;
          var L = values.L;
          var A = values.A;
        }
      var textcolor = colorUtils.colorBrightness(H,S,L) <= 50 ?
        '.tabbrowser-tab[selected] .tab-text,\
        #navigator-toolbox > toolbar:not(#TabsToolbar):not(#toolbar-menubar) toolbarbutton {\
          color: white !important;\
          text-shadow: none !important;\
      }' : '';
    this._ref = newval ? sheetService.load(sheetService.getEmbedCSSURL(
'#main-window:not([disablechrome]) #navigator-toolbox[tabsontop=true] > toolbar:not(#TabsToolbar):not(#toolbar-menubar):not(#nav-bar) {\
  background-color:' + colorUtils.HSLA(H,S,L - 10,A) + ' !important;\
}\
#main-window:not([disablechrome]):not(:-moz-any([stratiform-tabbar-style="s40"],[stratiform-tabbar-style="s50"],[stratiform-tabbar-style="soft7"])) #TabsToolbar .tabbrowser-tab[selected] > .tab-stack {\
  background-image: -moz-linear-gradient(hsla(0,0%,100%,.4), hsla(0,0%,100%,0)) !important;\
  background-color: ' + colorUtils.HSLA(H,S,L,100) + ' !important;\
}\
#main-window:not([disablechrome]):-moz-any([stratiform-tabbar-style="s40"],[stratiform-tabbar-style="s50"],[stratiform-tabbar-style="soft7"]) #TabsToolbar .tabbrowser-tab[selected] .tab-background {\
  background-color: ' + colorUtils.HSLA(H,S,L,100) + ' !important;\
  border-color: ' + colorUtils.HSLA(H,S,L,0) + ' !important;\
  border-bottom-color: ' + colorUtils.HSLA(H,S,L,100) + ' !important;\
}\
#main-window:not([disablechrome]) #navigator-toolbox[tabsontop=true] > #nav-bar,\
#main-window:not([disablechrome]) #navigator-toolbox[tabsontop=true]:not([customizing]) > #nav-bar[collapsed=true] + toolbar:not(#TabsToolbar):not(#toolbar-menubar):not(#nav-bar),\
#main-window:not([disablechrome]) #navigator-toolbox[tabsontop=true]:not([customizing]) > #nav-bar[collapsed=true] + toolbar[collapsed=true] ~ #customToolbars ~ #PersonalToolbar {\
  background-image: -moz-linear-gradient(' + colorUtils.HSLA(H,S,L,100) + ', ' + colorUtils.HSLA(H,S,L - 10,A) + ') !important;\
  background-color: transparent !important;\
}\
#browser-bottombox{\
  background-clip: padding-box !important;\
  background-color:' + colorUtils.HSLA(H,S,L,100) + '!important;\
}\
#main-window:-moz-any([stratiform-tabbar-style="squared"],:not([stratiform-tabbar-style])) #navigator-toolbox::after {\
  background-color: transparent !important;\
  background-image: -moz-linear-gradient(rgba(26,26,26,.4), rgba(26,26,26,.4)) !important;\
}\
}\
    ' + textcolor), null) : null;
      } catch(e) {}
    }
  }
};

const bookmarkStar = {
  change: function(newval) {
    var urlbaricons = $('urlbar-icons');
    var star = $('star-button');
    var urlbar = $('urlbar');
    if (newval) {
      var shbox = $('LeftSideStar-hbox');
      if (!shbox) {
        var shbox = mainWindow.document.createElement('box');
        shbox.setAttribute('id','LeftSideStar-hbox');
        shbox.setAttribute('align','center');
      }
      shbox.appendChild(star);
      urlbar.insertBefore(shbox,($('notification-popup-box')));
      urlbar.setAttribute('leftsidestar',true);
    }  
    else if (star.parentNode.id != 'urlbar-icons') {
      urlbaricons.insertBefore(star,urlbaricons.lastChild);
      urlbar.removeAttribute('leftsidestar');
    }
  }
};

const tabstitlebar = {
  _ref: null,
  onWindowResize: function(action) {
    try {
      var tabs = $('TabsToolbar');
	  var customizeMode = $('navigator-toolbox').customizing;
      if (mainWindow.windowState == 3) {
        var appmenu = $('appmenu-button').getBoundingClientRect().width;
        var caption = $('titlebar-buttonbox').getBoundingClientRect().width;
      }
      var menubar = $('toolbar-menubar');
      var tabsTop = $('main-window').getAttribute('tabsontop')=='true';
      var items = [tabs,menubar];

      if (mainWindow.windowState==3 | (mainWindow.windowState==3 && action =='enable')) {
        for (var i in items) {
          items[i].style.MozPaddingEnd = caption + 18 + 'px';
          items[i].style.MozPaddingStart = appmenu + 4 + 'px';
        }
      }
      if (mainWindow.windowState==1 | action =='disable' | !tabsTop | mainWindow.windowState==4 | customizeMode) {
        for (var i in items) {
          if (items[i].hasAttribute('style')) {
            items[i].style.MozPaddingEnd = '';
            items[i].style.MozPaddingStart = '';
          }
        }
      }
    } catch(e) {}
  },
  change: function(newval) {
    if (this._ref) {
      sheetService.disable(this._ref);
      delete this._ref;
    }
    if (newval) {
      this.onWindowResize('enable');
      mainWindow.addEventListener('resize', this.onWindowResize, false);
      this._ref = newval ? sheetService.load(sheetService.getEmbedCSSURL('\
        #main-window[chromemargin^="0,"][sizemode="normal"]:not([inFullscreen="true"])[tabsontop="true"] #navigator-toolbox:not([customizing="true"]) {\
          margin-top:' + (-30 + prefBranch.getIntPref('tabstitlebar-distance')) + 'px !important;\
        }\
        #main-window[chromemargin^="0,"][sizemode="normal"]:not([inFullscreen="true"])[tabsontop="true"] :-moz-any(#appmenu-button-container,#titlebar-buttonbox-container) {\
          position: relative !important;\
        }\
        #main-window[chromemargin^="0,"][sizemode="normal"] #toolbar-menubar:not([autohide="true"]),\
        #main-window[chromemargin^="0,"][sizemode="normal"] #TabsToolbar[tabsontop="true"],\
        #main-window[chromemargin^="0,"][sizemode="normal"] #navigator-toolbox[tabsontop="false"] > #nav-bar,\
        #main-window[chromemargin^="0,"][sizemode="normal"] #nav-bar + #customToolbars + #PersonalToolbar[collapsed="true"] + #TabsToolbar[tabsontop="false"]:last-child,\
        #main-window[chromemargin^="0,"][sizemode="normal"] #navigator-toolbox > toolbar:not(#toolbar-menubar):-moz-lwtheme {\
          -moz-binding: url("chrome://global/content/bindings/toolbar.xml#toolbar-drag");\
        }}\
      '), null) : null;
    }
    else {
      this.onWindowResize('disable');
      mainWindow.removeEventListener('resize', this.onWindowResize, false);
    }
  }
};

const tabStyleButtons = {
  change: function(newval) {
    var tabstoolbar = $('TabsToolbar');
    var list = tabstoolbar.getElementsByClassName('toolbarbutton-1');
    var whitelist = prefBranch.getCharPref('tabstyled-buttons-whitelist').split(',');
    switch (newval) {
      case true:
        tabstoolbar.setAttribute('stratiform-tabstyle-buttons',true);
        for (var i = 0; i < list.length; i ++) {
          if (whitelist.indexOf(list[i].id) != -1) {
            list[i].setAttribute('stratiform-tabstyled', true);
			if (list[i].parentNode.tagName == 'toolbaritem')
			  list[i].parentNode.setAttribute('stratiform-tabstyled',true);
	      }
        }
        mainWindow.addEventListener("aftercustomization", this.enable, false);
        break;
      case false:
        tabstoolbar.removeAttribute('stratiform-tabstyle-buttons');
        for (var i = 0; i < list.length; i ++) {
          list[i].removeAttribute('stratiform-tabstyled');
        }
        mainWindow.removeEventListener("aftercustomization", this.disable, false);
    }
    mainWindow.addEventListener("beforecustomization", this.disable, false);
  },
  enable: function() {
    tabStyleButtons.change(true);
  },
  disable: function() {
    tabStyleButtons.change(false);
  }
};

const conditionalFowardDisabler = {
  change: function(newval) {
    var insertLocation = $('nav-bar');
    var currentSet = insertLocation.getAttribute('currentset');
    var urlbar_container = $('urlbar-container');
    var combined_bf = $('unified-back-forward-button');
    var attVal = $('main-window').getAttribute('stratiform-toolbar-style');
    var prefVal = prefBranch.getBoolPref('disable-conditionalForward');
    if (newval == 'rubix' || newval == 'soft7' || prefVal == true && newval != 'disableConForward') {
      try {
        if (currentSet.indexOf('stratiform-conditional_forward-disabler') == -1 && combined_bf.nextSibling.id == 'urlbar-container') {
          var newSet = currentSet.replace(',urlbar-container',',stratiform-conditional_forward-disabler,urlbar-container');
          insertLocation.setAttribute('currentset', newSet);
          insertLocation.currentSet = newSet;
          document.persist('nav-bar', 'currentset');
          BrowserToolboxCustomizeDone(true);
        }
      } catch(e) {}
    }
    else
    if (prefVal == false && attVal != 'rubix' && attVal != 'soft7' || newval == 'disableConForward') {
      if (newval != 'rubix' && newval != 'soft7') {
        if (currentSet.indexOf('stratiform-conditional_forward-disabler') != -1) {
          var newSet = currentSet.replace(',stratiform-conditional_forward-disabler,urlbar-container',',urlbar-container');
          insertLocation.setAttribute('currentset', newSet);
          insertLocation.currentSet = newSet;
          document.persist('nav-bar', 'currentset');
          BrowserToolboxCustomizeDone(true);
        }
      }
    }
  },
  removeCon: function() {
    conditionalFowardDisabler.change('disableConForward');
  },
  enableCon: function() {
    var mw = $('main-window').getAttribute('stratiform-toolbar-style');
    conditionalFowardDisabler.change(mw);
  }
};

const Prefs = {

  _prefs: {
    'appmenu-label': {
      binding: appmenuLabel
    },
    'toolbar-style': {
      binding: conditionalFowardDisabler,
      att: 'stratiform-toolbar-style',
      's40': {
        css: 'toolbar/strata40/toolbar.css'
      },
      '7native': {
        css: 'toolbar/7native/toolbar.css'
      },
      'untitled': {
        css: 'toolbar/untitled/toolbar.css'
      },
      'wave': {
        css: 'toolbar/wave/toolbar.css'
      },
      'rubix': {
        css: 'toolbar/rubix/toolbar.css'
      },
      'soft7': {
        css: 'toolbar/soft7/toolbar.css'
      }
    },
    'appmenu-button-style': {
      att: 'stratiform-appmenubutton-style',
      'squared': {
        css: 'appmenu-button/squared/appmenu.css'
      },
      'soft7': {
        css: 'appmenu-button/soft7/appmenu.css'
      }
    },
    'toolbar-radius': {
      binding: toolbarBorderRadius
    },
    'tabbar-style': {
      att: 'stratiform-tabbar-style',
      's40': {
        strictCompat: 'true',
        css: 'tabbar/strata40/tabbar.css'
      },
      'squared': {
        strictCompat: 'true',
        css: 'tabbar/squared/tabbar.css'
      },
      'wave': {
        strictCompat: 'true',
        css: 'tabbar/wave/tabbar.css'
      },
      's50': {
        strictCompat: 'true',
        css: 'tabbar/s50/tabbar.css'
      },
      'soft7': {
        strictCompat: 'true',
        css: 'tabbar/soft7/tabbar.css'
      },
    },
    'textfield-style': {
      's40': {
        css: 'text-fields/strata40/text-fields.css'
      },
      'untitled': {
        css: 'text-fields/untitled/text-fields.css'
      },
      '7native': {
        css: 'text-fields/7native/text-fields.css'
      },
      'wave': {
        css: 'text-fields/wave/text-fields.css'
      },
      'soft7': {
        css: 'text-fields/soft7/text-fields.css'
      }
    },
    'glyph-style': {
      's40': {
        css: 'glyphs/s40/glyph.css'
      },
      '7native': {
        css: 'glyphs/7native/glyph.css'
      },
      'untitled': {
        css: 'glyphs/untitled/glyph.css'
      },
      'wave': {
        css: 'glyphs/wave/glyph.css'
      },
      'fusion': {
        css: 'glyphs/fusion/glyph.css'
      },
      'soft7': {
        css: 'glyphs/soft7/glyph.css'
      }
    },
    'appmenu-style': {
      'always': {
        css: 'appmenu-button/slimbutton.css'
      },
      'restored': {
        css: 'appmenu-button/slimbutton-restored.css'
      },
      'maxonly': {
        css: 'appmenu-button/slimbutton-max.css'
      }
    },
    'addonbar-style': {
      's40': {
        css: 'addon-bar/strata40/addonbar.css'
      }
    },
    'statuspopup-style': {
      'native': {
        css: 'status-popup/native/status.css'
      },
      'windows7': {
        css: 'status-popup/windows7/status.css'
      }
    },
    'toolbar-bg': {
      binding: toolbarBackgroundColor
    },
    'inactivetab-bg': {
      strictCompat: 'true',
      binding: inactiveTabColor
    },
    'selectedtab-bg': {
      strictCompat: 'true',
      binding: activeTabColor
    },
    'appmenu-bg': {
      binding: appmenuColor
    },
    'tab-width': {
      binding: tabWidth
    },
    'tab-height': {
      strictCompat: 'true',
      att: 'stratiform-tabbar-height',
      binding: tabHeight
    },
    'tabstyle-buttons': {
      strictCompat: 'true',
      binding: tabStyleButtons
    },
    'leftsidestar': {
      binding: bookmarkStar
    },
    'thirdparty-customicons': {
      att: 'stratiform-custom-icons',
      true: {},
      false: {},
    },
    'disable-conditionalForward': {
      binding: conditionalFowardDisabler
    },
    'tabstitlebar': {
      binding: tabstitlebar
    },
  },

  getPrefValue: function(key) {
    switch (prefBranch.getPrefType(key)) {
      case prefBranch.PREF_INT:
        return prefBranch.getIntPref(key);
      case prefBranch.PREF_BOOL:
        return prefBranch.getBoolPref(key);
      case prefBranch.PREF_STRING:
        return prefBranch.getCharPref(key);
    }
    return null;
  },

  enableNew: function(pref, val) {
    if ('att' in pref && val)
      document.documentElement.setAttribute(pref.att, val);
    if ('binding' in pref && !pref.strictCompat)
      pref.binding.change(val, pref);
    else if ('binding' in pref && pref.strictCompat) {
      if (!compatibilityMode._enabled)
        pref.binding.change(val, pref);
    }
    pref.last = val;
    if (!(val in pref))
      return;
    var valOpt = pref[val];
    if ('css' in valOpt && !valOpt.strictCompat)
      pref.ref = sheetService.load(valOpt.css, BaseURI);
    else if ('css' in valOpt && valOpt.strictCompat) {
      if (!compatibilityMode._enabled)
        pref.ref = sheetService.load(valOpt.css, BaseURI);
    }
    if ('binding' in valOpt && val == true)
      valOpt.binding.enable(pref.ref, val);
  },

  disableLast: function(pref) {
    if (!('last' in pref) || !(pref.last in pref))
      return;
    if (pref.ref) {
      sheetService.disable(pref.ref);
      delete pref.ref;
    }
    if ('binding' in pref[pref.last])
      pref[pref.last].binding.disable();
    if ('att' in pref)
      document.documentElement.removeAttribute(pref.att);
  },

  onPrefChange : {
    observe: function(subject, topic, key) {
      if (!(key in Prefs._prefs))
        return;
      var pref = Prefs._prefs[key];
      var val = Prefs.getPrefValue(key);
      if (pref.last == val)
        return;
      Prefs.disableLast(pref);
      if (val != null)
        Prefs.enableNew(pref, val);
    }
  },

  enable: function() {
    for (var key in this._prefs) {
      var val = this.getPrefValue(key);
      if (val != null)
        this.enableNew(this._prefs[key], val);
    }
    prefBranch.addObserver('', this.onPrefChange, false);
  },

  disable: function() {
    for each (var pref in this._prefs)
      this.disableLast(pref);
    prefBranch.removeObserver('', this.onPrefChange);
  }

};

// Listen for add-on uninstall so we can remove the conditional forward disabler from currentset
let listener = {
  onUninstalling: function(addon) {
    if (addon.id == "Stratiform@SoapySpew") {
      conditionalFowardDisabler.removeCon();
    }
  },
  onDisabling: function(addon) {
    if (addon.id == "Stratiform@SoapySpew") {
      conditionalFowardDisabler.removeCon();
    }
  },
  onOperationCancelled: function(addon) {
    if (addon.id == "Stratiform@SoapySpew") {
      conditionalFowardDisabler.enableCon();
    }
  }
}

function init() {
  window.removeEventListener('load', init, false);
  compatibilityMode.check();
  Prefs.enable();
}

function uninit() {
  Prefs.disable();
}

function afterPaint() {
  window.removeEventListener('MozAfterPaint', afterPaint, false);
  Stratiform.firstRun();
  if (prefGeneral.getCharPref('extensions.enabledAddons').indexOf('{7067a92c-1db4-4e5e-869c-25f841287f8b}')!= -1)
    sheetService.load('showExecuteJS.css',BaseURI);
  window.addEventListener('beforecustomization', conditionalFowardDisabler.removeCon, false);
  window.addEventListener('aftercustomization', conditionalFowardDisabler.enableCon, false);
  var appMenu = $('appmenu-popup');
  if (appMenu)
    appMenu.addEventListener('popupshowing', function() {
                                               var mi = $('stratiform_splitmenu');
                                               if (mi)
                                                 mi.hidden = !prefBranch.getBoolPref('show-in-firefoxMenu');
                                             }, false);
}

window.Stratiform = {
  openTab: function(url) {
    if (url == 'donate')
      mainWindow.switchToTabHavingURI('https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=8Z74B76F79LTJ&lc=US&item_name=Stratiform&currency_code=USD&bn=PP%2dDonationsBF%3abtn_donate_LG%2egif%3aNonHosted',true);
	else
	  mainWindow.switchToTabHavingURI(url,true);
  },
  firstRun: function() {
    try {
      if (prefBranch.getBoolPref('firstrun')) {
        var addonbar = $('addon-bar');
        if (addonbar.currentSet.indexOf('stratiform-options-button') == -1) {
          if (addonbar.getAttribute('collapsed') == 'true')
            var insertLocation = $('nav-bar');
          else
            var insertLocation = addonbar;
          // If insertLocation hasn't been customized copy the defaultset to currentset and add our button.
          if (!insertLocation.hasAttribute('currentset')) {
            var ds = insertLocation.getAttribute('defaultset');
            insertLocation.setAttribute('currentset',ds);
          }
          var button = ',stratiform-options-button';
          var curSet = insertLocation.getAttribute('currentset');
          insertLocation.setAttribute('currentset', curSet + button);
          insertLocation.currentSet = curSet + button;
          document.persist(insertLocation.id, 'currentset');
          BrowserToolboxCustomizeDone(true);
        }
        setTimeout(
          function() {
            Stratiform.openTab('about:stratiform');
          },250);
        prefBranch.setBoolPref('firstrun',false);
      }
    } catch(e) {}
  }
};
AddonManager.addAddonListener(listener);
window.addEventListener('load', init, false);
window.addEventListener('MozAfterPaint', afterPaint, false);
//window.addEventListener('unload', uninit, false);
})();