'use strict';
var EXPORTED_SYMBOLS = ['StratiformCompatibilityMode','StratiformColorUtils','StratiformUtilities'];
const Cc = Components.classes, Ci = Components.interfaces;

Components.utils.import('resource://gre/modules/Services.jsm');

var StratiformUtilities = {
  prefService: function() {
    return Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefService);
  },
  prefBranch: function() {
    return this.prefService().getBranch('extensions.stratiform.').QueryInterface(Ci.nsIPrefBranch2);
  },
  prefGeneral: function() {
    return this.prefService().QueryInterface(Ci.nsIPrefBranch2);
  },
  ssService: function() {
    return Cc['@mozilla.org/content/style-sheet-service;1'].getService(Ci.nsIStyleSheetService);
  },
  ioService: function() {
    return Cc['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);
  },
  mainWindow: function() {
    return Services.wm.getMostRecentWindow('navigator:browser');
  }
}

var StratiformCompatibilityMode = {
  _enabled: false,
  check: function() {
    this.incompatList = ['{1cff04ef-0c75-4621-ba2a-2efb77346996}','{dc572301-7619-498c-a57d-39143191b318}','{0545b830-f0aa-4d7e-8820-50a4629a56fe}','fabtab@captaincaveman.nl','Seven@SoapyHamHocks','verticaltabs@philikon.de','treestyletab@piro.sakura.ne.jp','{752a85d4-68d6-48ae-ab7d-6640f5f75d85}','Australis@SoapyHamHocks','tabutils@ithinc.cn'];
    this.enabledAddons = StratiformUtilities.prefGeneral().getCharPref('extensions.enabledAddons');
    this.disableCheck = StratiformUtilities.prefBranch().getBoolPref('disable-strict-compatibility');
    if (!this.disableCheck) {
      for (var i in this.incompatList) {
        if (this.enabledAddons.indexOf(this.incompatList[i]) !=-1) {
          this._enabled = true;
          StratiformUtilities.mainWindow().document.getElementById('main-window').setAttribute('stratiform-compatibility-mode',true);
          break;
        }
      }
    }
  }
}

var StratiformColorUtils = {
  HSLA: function(H,S,L,A) {
    return 'hsla(' + H + ',' + S + '%,' + L + '%,' + A / 100 +')';
  },
  hexToRgb: function(hex) {
    if (hex) {
      if (hex.length==3)
        var hex = hex.substring(0,1) + hex.substring(0,1) + hex.substring(1,2) + hex.substring(1,2) + hex.substring(2,3) + hex.substring(2,3);
      return [parseInt((hex.substring(0,2)),16), parseInt((hex.substring(2,4)),16), parseInt((hex.substring(4,6)),16)];
    }
  },
  rgbToHsl: function(r,g,b) {
    var r,g,b;
    r /= 255, g /= 255, b /= 255;
    var max = Math.max(r, g, b), min = Math.min(r, g, b);
    var h, s, l = (max + min) / 2;
    if(max == min)
      h = s = 0;
    else {
      var d = max - min;
      s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
      switch(max) {
        case r: h = (g - b) / d + (g < b ? 6 : 0); break;
        case g: h = (b - r) / d + 2; break;
        case b: h = (r - g) / d + 4; break;
      }
      h /= 6;
    }
    return [Math.round(h*360), Math.round(s*100), Math.round(l*100)];
  },
  hue2rgb: function(p, q, t) {
    if (t < 0)
      t += 1;
    if (t > 1)
      t -= 1;
    if (t < 1/6)
      return p + (q - p) * 6 * t;
    if (t < 1/2)
      return q;
    if (t < 2/3)
      return p + (q - p) * (2/3 - t) * 6;
    return p;
  },
  hslToRgb: function(h, s, l) {
    var r, g, b;
    var h = h / 360;
    var s = s / 100;
    var l = l /100;
    if(s == 0) {
      r = g = b = l;
    } else {
      var q = l < 0.5 ? l * (1 + s) : l + s - l * s;
      var p = 2 * l - q;
      r = this.hue2rgb(p, q, h + 1/3);
      g = this.hue2rgb(p, q, h);
      b = this.hue2rgb(p, q, h - 1/3);
    }
    return [r * 255, g * 255, b * 255];
  },
  // Thanks to http://zoltanb.co.uk for the algorithm.
  colorBrightness: function(h,s,l) {
    var rgb = this.hslToRgb(h,s,l);
    return Math.round((Math.sqrt(rgb[0] * rgb[0] * .241 + rgb[1] * rgb[1] * .691 + rgb[2] * rgb[2] * .068) / 255) * 100);
  }
}